@@ -249,8 +249,13 @@ static char* DKIMErrorStrings[-1-DKIM_MAX_ERROR] = {
 	"DKIM_NO_VALID_SIGNATURES",
 	"DKIM_BODY_HASH_MISMATCH",
 	"DKIM_SELECTOR_ALGORITHM_MISMATCH",
-	"DKIM_STAT_INCOMPAT"
-
+	"DKIM_STAT_INCOMPAT",
+	"DKIM_UNSIGNED_FROM",
+	"DKIM_OUT_OF_MEMORY",
+	"DKIM_INVALID_CONTEXT",
+	"DKIM_NO_SENDER",
+	"DKIM_BAD_PRIVATE_KEY",
+	"DKIM_BUFFER_TOO_SMALL",
 };
 
 
