@@ -51,14 +51,7 @@ extern "C" {
 #define DKIM_SIGN_RELAXED_SIMPLE	MAKELONG(DKIM_CANON_SIMPLE,DKIM_CANON_RELAXED)
 
 // DKIM Error codes
-#define DKIM_SUCCESS			0	// operation successful
-#define DKIM_OUT_OF_MEMORY		1	// memory allocation failed
-#define DKIM_INVALID_CONTEXT	2	// DKIMContext structure invalid for this operation
-#define DKIM_NO_SENDER			3	// Could not find From: or Sender: header in message
-#define DKIM_BAD_PRIVATE_KEY	4	// Could not parse private key
-#define DKIM_BUFFER_TOO_SMALL	5	// Buffer passed in is not large enough
-
-// DKIM Verification Error codes
+#define DKIM_SUCCESS						0		// operation successful
 #define DKIM_FAIL							-1		// verify error: message is suspicious
 #define DKIM_BAD_SYNTAX						-2		// signature error: DKIM-Signature could not parse or has bad tags/values
 #define DKIM_SIGNATURE_BAD					-3		// signature error: RSA verify failed
@@ -76,7 +69,13 @@ extern "C" {
 #define DKIM_BODY_HASH_MISMATCH				-15		// sigature verify error: message body does not hash to bh value
 #define DKIM_SELECTOR_ALGORITHM_MISMATCH	-16		// signature error: selector h= doesn't match signature a=
 #define DKIM_STAT_INCOMPAT					-17		// signature error: incompatible v=
-#define DKIM_MAX_ERROR						-18		// set this to 1 greater than the highest error code (but negative)
+#define DKIM_UNSIGNED_FROM					-18		// signature error: not all message's From headers in signature
+#define DKIM_OUT_OF_MEMORY					-20		// memory allocation failed
+#define DKIM_INVALID_CONTEXT				-21		// DKIMContext structure invalid for this operation
+#define DKIM_NO_SENDER						-22		// signing error: Could not find From: or Sender: header in message
+#define DKIM_BAD_PRIVATE_KEY				-23		// signing error: Could not parse private key
+#define DKIM_BUFFER_TOO_SMALL				-24		// signing error: Buffer passed in is not large enough
+#define DKIM_MAX_ERROR						-25		// set this to 1 greater than the highest error code (but negative)
 
 // DKIM_SUCCESS									// verify result: all signatures verified
 												// signature result: signature verified
@@ -126,17 +125,19 @@ typedef struct DKIMSignOptions_t
 typedef struct DKIMVerifyOptions_t
 {
 	DKIMDNSCALLBACK pfnSelectorCallback;	// selector record callback
-	DKIMDNSCALLBACK pfnPracticesCallback;	// SSP record callback
+	DKIMDNSCALLBACK pfnPracticesCallback;	// ADSP record callback
 	int nHonorBodyLengthTag;				// 0 = ignore l= tag, 1 = use l= tag to limit the amount of body verified
-	int nCheckPractices;					// 0 = use default (unknown) practices, 1 = request and use sender's signing practices
+	int nCheckPractices;					// 0 = use default (unknown) practices, 1 = request and use author domain signing practices
 	int nSubjectRequired;					// 0 = subject is required to be signed, 1 = not required
-	int nSaveCanonicalizedData;             // 0 = canonicalized data is not saved, 1 = canonicalized data is saved
+	int nSaveCanonicalizedData;				// 0 = canonicalized data is not saved, 1 = canonicalized data is saved
+	int nAllowUnsignedFromHeaders;			// 0 = From headers not included in the signature are not allowed, 1 = allowed
 } DKIMVerifyOptions;
 
 typedef struct DKIMVerifyDetails_t
 {
 	char *szSignature;
-	char *DNS;
+	char *szSignatureDomain;
+	char *szIdentityDomain;
 	char *szCanonicalizedData;
 	int nResult;
 } DKIMVerifyDetails;
